@@ -6,8 +6,8 @@ use dashmap::DashMap;
 use deno_core::anyhow::anyhow;
 use deno_core::error::AnyError;
 use deno_core::serde_json;
-use deno_core::url::Url;
 use deno_npm::npm_rc::NpmRc;
+use deno_npm::npm_rc::NpmRegistryUrl;
 use deno_npm::resolution::NpmVersionResolver;
 use deno_semver::Version;
 use deno_semver::package::PackageNv;
@@ -59,7 +59,7 @@ impl PackageSearchApi for CliNpmSearchApi {
     if let Some(names) = self.search_cache.get(query) {
       return Ok(names.clone());
     }
-    let mut search_url = npm_registry_url().join("-/v1/search")?;
+    let mut search_url = npm_registry_url().url.join("-/v1/search")?;
     search_url
       .query_pairs_mut()
       .append_pair("text", &format!("{} boost-exact:false", query));
@@ -124,9 +124,9 @@ fn parse_npm_search_response(source: &str) -> Result<Vec<String>, AnyError> {
 }
 
 // this is buried here because generally you want to use the ResolvedNpmRc instead of this.
-fn npm_registry_url() -> &'static Url {
-  static NPM_REGISTRY_DEFAULT_URL: Lazy<Url> =
-    Lazy::new(|| deno_resolver::npmrc::npm_registry_url(&CliSys::default()));
+fn npm_registry_url() -> &'static NpmRegistryUrl {
+  static NPM_REGISTRY_DEFAULT_URL: Lazy<NpmRegistryUrl> =
+    Lazy::new(|| NpmRegistryUrl::for_npm(&CliSys::default()));
 
   &NPM_REGISTRY_DEFAULT_URL
 }
