@@ -28,6 +28,7 @@ use deno_core::RcRef;
 use deno_core::Resource;
 use deno_core::ResourceId;
 use deno_core::ToJsBuffer;
+use deno_core::convert::Uint8Array;
 use deno_core::futures::StreamExt;
 use deno_core::op2;
 use deno_core::serde_v8::AnyValue;
@@ -209,7 +210,7 @@ pub enum KvErrorKind {
   InvalidRange,
 }
 
-#[op2(async, stack_trace)]
+#[op2(stack_trace)]
 #[smi]
 async fn op_kv_database_open<DBH>(
   state: Rc<RefCell<OpState>>,
@@ -348,7 +349,7 @@ type SnapshotReadRange = (
   Option<ByteString>,
 );
 
-#[op2(async)]
+#[op2]
 #[serde]
 async fn op_kv_snapshot_read<DBH>(
   state: Rc<RefCell<OpState>>,
@@ -434,12 +435,11 @@ impl<QMH: QueueMessageHandle + 'static> Resource for QueueMessageResource<QMH> {
   }
 }
 
-#[op2(async)]
-#[serde]
+#[op2]
 async fn op_kv_dequeue_next_message<DBH>(
   state: Rc<RefCell<OpState>>,
   #[smi] rid: ResourceId,
-) -> Result<Option<(ToJsBuffer, ResourceId)>, KvError>
+) -> Result<Option<(Uint8Array, ResourceId)>, KvError>
 where
   DBH: DatabaseHandler + 'static,
 {
@@ -519,7 +519,7 @@ enum WatchEntry {
   Unchanged,
 }
 
-#[op2(async)]
+#[op2]
 #[serde]
 async fn op_kv_watch_next(
   state: Rc<RefCell<OpState>>,
@@ -572,7 +572,7 @@ async fn op_kv_watch_next(
   Ok(Some(entries))
 }
 
-#[op2(async)]
+#[op2]
 async fn op_kv_finish_dequeued_message<DBH>(
   state: Rc<RefCell<OpState>>,
   #[smi] handle_rid: ResourceId,
@@ -884,7 +884,7 @@ fn decode_selector_and_cursor(
   Ok((first_key, last_key))
 }
 
-#[op2(async)]
+#[op2]
 #[string]
 async fn op_kv_atomic_write<DBH>(
   state: Rc<RefCell<OpState>>,
