@@ -22,13 +22,16 @@ use deno_core::AsyncRefCell;
 use deno_core::AsyncResult;
 use deno_core::BufMutView;
 use deno_core::BufView;
+use deno_core::FromV8;
 use deno_core::GarbageCollected;
 use deno_core::JsBuffer;
 use deno_core::OpState;
 use deno_core::RcRef;
 use deno_core::Resource;
 use deno_core::ResourceId;
+use deno_core::ToV8;
 use deno_core::WriteOutcome;
+use deno_core::convert::Uint8Array;
 use deno_core::error::ResourceError;
 use deno_core::op2;
 use deno_error::JsError;
@@ -46,8 +49,6 @@ use quinn::crypto::rustls::QuicServerConfig;
 use quinn::rustls::client::ClientSessionMemoryCache;
 use quinn::rustls::client::ClientSessionStore;
 use quinn::rustls::client::Resumption;
-use serde::Deserialize;
-use serde::Serialize;
 
 use crate::DefaultTlsOptions;
 use crate::UnsafelyIgnoreCertificateErrors;
@@ -122,33 +123,33 @@ pub enum QuicError {
   Other(#[from] JsErrorBox),
 }
 
-#[derive(Serialize, Deserialize)]
+#[derive(serde::Serialize, serde::Deserialize)]
 #[serde(rename_all = "camelCase")]
 struct CloseInfo {
   close_code: u64,
   reason: String,
 }
 
-#[derive(Debug, Deserialize, Serialize)]
+#[derive(Debug, FromV8, ToV8)]
 struct Addr {
   hostname: String,
   port: u16,
 }
 
-#[derive(Deserialize)]
-#[serde(rename_all = "camelCase")]
+#[derive(FromV8)]
 struct ListenArgs {
   alpn_protocols: Option<Vec<String>>,
 }
 
-#[derive(Deserialize, Default, PartialEq)]
-#[serde(rename_all = "camelCase")]
+#[derive(FromV8, Default, PartialEq)]
 struct TransportConfig {
   keep_alive_interval: Option<u64>,
   max_idle_timeout: Option<u64>,
   max_concurrent_bidirectional_streams: Option<u32>,
   max_concurrent_unidirectional_streams: Option<u32>,
+  #[from_v8(serde)]
   preferred_address_v4: Option<SocketAddrV4>,
+  #[from_v8(serde)]
   preferred_address_v6: Option<SocketAddrV6>,
   congestion_control: Option<String>,
 }
@@ -229,7 +230,7 @@ unsafe impl GarbageCollected for EndpointResource {
 #[cppgc]
 pub(crate) fn op_quic_endpoint_create(
   state: Rc<RefCell<OpState>>,
-  #[serde] addr: Addr,
+  #[scoped] addr: Addr,
   can_listen: bool,
 ) -> Result<EndpointResource, QuicError> {
   let addr = resolve_addr_sync(&addr.hostname, addr.port)?
@@ -272,7 +273,6 @@ pub(crate) fn op_quic_endpoint_create(
 }
 
 #[op2]
-#[serde]
 pub(crate) fn op_quic_endpoint_get_addr(
   #[cppgc] endpoint: &EndpointResource,
 ) -> Result<Addr, QuicError> {
@@ -317,8 +317,8 @@ unsafe impl GarbageCollected for ListenerResource {
 #[cppgc]
 pub(crate) fn op_quic_endpoint_listen(
   #[cppgc] endpoint: &EndpointResource,
-  #[serde] args: ListenArgs,
-  #[serde] transport_config: TransportConfig,
+  #[scoped] args: ListenArgs,
+  #[scoped] transport_config: TransportConfig,
   #[cppgc] keys: &TlsKeysHolder,
 ) -> Result<ListenerResource, QuicError> {
   if !endpoint.can_listen {
@@ -385,7 +385,7 @@ unsafe impl GarbageCollected for IncomingResource {
   }
 }
 
-#[op2(async)]
+#[op2]
 #[cppgc]
 pub(crate) async fn op_quic_listener_accept(
   #[cppgc] resource: &ListenerResource,
@@ -416,7 +416,6 @@ pub(crate) fn op_quic_incoming_local_ip(
 }
 
 #[op2]
-#[serde]
 pub(crate) fn op_quic_incoming_remote_addr(
   #[cppgc] incoming_resource: &IncomingResource,
 ) -> Result<Addr, QuicError> {
@@ -458,11 +457,11 @@ fn quic_incoming_accept(
   }
 }
 
-#[op2(async)]
+#[op2]
 #[cppgc]
 pub(crate) async fn op_quic_incoming_accept(
   #[cppgc] incoming_resource: &IncomingResource,
-  #[serde] transport_config: Option<TransportConfig>,
+  #[scoped] transport_config: Option<TransportConfig>,
 ) -> Result<ConnectionResource, QuicError> {
   let connecting = quic_incoming_accept(incoming_resource, transport_config)?;
   let conn = connecting.await?;
@@ -473,7 +472,7 @@ pub(crate) async fn op_quic_incoming_accept(
 #[cppgc]
 pub(crate) fn op_quic_incoming_accept_0rtt(
   #[cppgc] incoming_resource: &IncomingResource,
-  #[serde] transport_config: Option<TransportConfig>,
+  #[scoped] transport_config: Option<TransportConfig>,
 ) -> Result<ConnectionResource, QuicError> {
   let connecting = quic_incoming_accept(incoming_resource, transport_config)?;
   match connecting.into_0rtt() {
@@ -521,8 +520,7 @@ unsafe impl GarbageCollected for ConnectingResource {
   }
 }
 
-#[derive(Deserialize)]
-#[serde(rename_all = "camelCase")]
+#[derive(FromV8)]
 struct ConnectArgs {
   addr: Addr,
   ca_certs: Option<Vec<String>>,
@@ -531,10 +529,10 @@ struct ConnectArgs {
   server_certificate_hashes: Option<Vec<CertificateHash>>,
 }
 
-#[derive(Deserialize)]
-#[serde(rename_all = "camelCase")]
+#[derive(FromV8)]
 struct CertificateHash {
   algorithm: String,
+  #[from_v8(serde)]
   value: JsBuffer,
 }
 
@@ -543,8 +541,8 @@ struct CertificateHash {
 pub(crate) fn op_quic_endpoint_connect(
   state: Rc<RefCell<OpState>>,
   #[cppgc] endpoint: &EndpointResource,
-  #[serde] args: ConnectArgs,
-  #[serde] transport_config: TransportConfig,
+  #[scoped] args: ConnectArgs,
+  #[scoped] transport_config: TransportConfig,
   #[cppgc] key_pair: &TlsKeysHolder,
 ) -> Result<ConnectingResource, QuicError> {
   state
@@ -622,7 +620,7 @@ pub(crate) fn op_quic_endpoint_connect(
   Ok(ConnectingResource(RefCell::new(Some(connecting))))
 }
 
-#[op2(async)]
+#[op2]
 #[cppgc]
 pub(crate) async fn op_quic_connecting_1rtt(
   #[cppgc] connecting: &ConnectingResource,
@@ -677,7 +675,6 @@ pub(crate) fn op_quic_connection_get_server_name(
 }
 
 #[op2]
-#[serde]
 pub(crate) fn op_quic_connection_get_remote_addr(
   #[cppgc] connection: &ConnectionResource,
 ) -> Result<Addr, QuicError> {
@@ -700,7 +697,7 @@ pub(crate) fn op_quic_connection_close(
   Ok(())
 }
 
-#[op2(async)]
+#[op2]
 #[serde]
 pub(crate) async fn op_quic_connection_closed(
   #[cppgc] connection: &ConnectionResource,
@@ -719,7 +716,7 @@ pub(crate) async fn op_quic_connection_closed(
   }
 }
 
-#[op2(async)]
+#[op2]
 pub(crate) async fn op_quic_connection_handshake(
   #[cppgc] connection: &ConnectionResource,
 ) {
@@ -826,8 +823,7 @@ impl Resource for RecvStreamResource {
   }
 }
 
-#[op2(async)]
-#[serde]
+#[op2]
 pub(crate) async fn op_quic_connection_accept_bi(
   #[cppgc] connection: &ConnectionResource,
   state: Rc<RefCell<OpState>>,
@@ -849,8 +845,7 @@ pub(crate) async fn op_quic_connection_accept_bi(
   }
 }
 
-#[op2(async)]
-#[serde]
+#[op2]
 pub(crate) async fn op_quic_connection_open_bi(
   #[cppgc] connection: &ConnectionResource,
   state: Rc<RefCell<OpState>>,
@@ -874,8 +869,7 @@ pub(crate) async fn op_quic_connection_open_bi(
   Ok((tx_rid, rx_rid))
 }
 
-#[op2(async)]
-#[serde]
+#[op2]
 pub(crate) async fn op_quic_connection_accept_uni(
   #[cppgc] connection: &ConnectionResource,
   state: Rc<RefCell<OpState>>,
@@ -898,8 +892,7 @@ pub(crate) async fn op_quic_connection_accept_uni(
   }
 }
 
-#[op2(async)]
-#[serde]
+#[op2]
 pub(crate) async fn op_quic_connection_open_uni(
   #[cppgc] connection: &ConnectionResource,
   state: Rc<RefCell<OpState>>,
@@ -924,7 +917,7 @@ pub(crate) async fn op_quic_connection_open_uni(
   Ok(rid)
 }
 
-#[op2(async)]
+#[op2]
 pub(crate) async fn op_quic_connection_send_datagram(
   #[cppgc] connection: &ConnectionResource,
   #[buffer] buf: JsBuffer,
@@ -933,13 +926,12 @@ pub(crate) async fn op_quic_connection_send_datagram(
   Ok(())
 }
 
-#[op2(async)]
-#[buffer]
+#[op2]
 pub(crate) async fn op_quic_connection_read_datagram(
   #[cppgc] connection: &ConnectionResource,
-) -> Result<Vec<u8>, QuicError> {
+) -> Result<Uint8Array, QuicError> {
   let data = connection.0.read_datagram().await?;
-  Ok(data.into())
+  Ok(Vec::from(data).into())
 }
 
 #[op2(fast)]
@@ -1102,8 +1094,7 @@ pub(crate) mod webtransport {
     Ok((settings_tx_rid, settings_rx_rid))
   }
 
-  #[op2(async)]
-  #[serde]
+  #[op2]
   pub(crate) async fn op_webtransport_connect(
     state: Rc<RefCell<OpState>>,
     #[cppgc] connection_resource: &ConnectionResource,
@@ -1166,8 +1157,7 @@ pub(crate) mod webtransport {
     ))
   }
 
-  #[op2(async)]
-  #[serde]
+  #[op2]
   pub(crate) async fn op_webtransport_accept(
     state: Rc<RefCell<OpState>>,
     #[cppgc] connection_resource: &ConnectionResource,
