@@ -234,6 +234,7 @@ deno_core::extension!(deno_node,
     ops::v8::op_v8_write_value,
     ops::vm::op_vm_create_script,
     ops::vm::op_vm_create_context,
+    ops::vm::op_vm_create_context_without_contextify,
     ops::vm::op_vm_script_run_in_context,
     ops::vm::op_vm_is_context,
     ops::vm::op_vm_compile_function,
@@ -516,6 +517,7 @@ deno_core::extension!(deno_node,
     "internal/streams/utils.js",
     "internal/test/binding.ts",
     "internal/timers.mjs",
+    "internal/tty.js",
     "internal/url.ts",
     "internal/util.mjs",
     "internal/util/colors.ts",
