@@ -99,7 +99,6 @@ impl GPUDevice {
   }
 
   #[getter]
-  #[global]
   fn features(
     &self,
     scope: &mut v8::PinScope<'_, '_>,
@@ -112,7 +111,6 @@ impl GPUDevice {
   }
 
   #[getter]
-  #[global]
   fn limits(&self, scope: &mut v8::PinScope<'_, '_>) -> v8::Global<v8::Object> {
     self.limits.get(scope, |_| {
       let limits = self.instance.device_limits(self.id);
@@ -121,7 +119,6 @@ impl GPUDevice {
   }
 
   #[getter]
-  #[global]
   fn adapter_info(
     &self,
     scope: &mut v8::PinScope<'_, '_>,
@@ -134,7 +131,6 @@ impl GPUDevice {
   }
 
   #[getter]
-  #[global]
   fn queue(&self, scope: &mut v8::PinScope<'_, '_>) -> v8::Global<v8::Object> {
     self.queue_obj.get(scope, |_| GPUQueue {
       id: self.queue,
@@ -522,7 +518,6 @@ impl GPUDevice {
     self.new_render_pipeline(descriptor)
   }
 
-  #[async_method]
   #[required(1)]
   #[cppgc]
   async fn create_compute_pipeline_async(
@@ -532,7 +527,6 @@ impl GPUDevice {
     self.new_compute_pipeline(descriptor)
   }
 
-  #[async_method]
   #[required(1)]
   #[cppgc]
   async fn create_render_pipeline_async(
@@ -685,7 +679,6 @@ impl GPUDevice {
   }
 
   #[getter]
-  #[global]
   fn lost(&self) -> v8::Global<v8::Promise> {
     self.lost_promise.clone()
   }
@@ -702,7 +695,6 @@ impl GPUDevice {
   }
 
   #[async_method(fake)]
-  #[global]
   fn pop_error_scope(
     &self,
     scope: &mut v8::PinScope<'_, '_>,
